/*
 * This file was written by Gabriel and Carl
 * 
 * It is designed to sense either voltage or current and 
 * drive the transistor of a voltage boost circuit to maintain
 * either constant current or constant voltage.
 */



#include <TimerOne.h>

#define PWM_OUT 10      //Set PWM pin
#define FEEDBACK  A0    //Set sense pin
#define MAX_DUTY 800    //Set upper limit of PWM for inductor saturation

float voltage;
float set_voltage=5.5;  //Set the target voltage for feedback ctrl
float error;
float pwm_duty;
float kp=300;           //Set the feedback gain

void setup() {
    // Initializes Timer1 to overflow in 25 microsecond periods
    // 1/period=frequency
    Timer1.initialize(5);
    // Enables PWM on pin 10 with 50% duty cycle
    // This function expects a 10 bit value for duty cycle. I.E. a value between 0 (which is 0%) and 1024 (which is 100%)
    Timer1.pwm(PWM_OUT, 50);
    Serial.begin(9600);
}

void loop() {
    delay(1);
    voltage = (float)analogRead(FEEDBACK) * 0.01466;    //Scale the input to a voltage
    error = set_voltage-voltage;                        //Calculate error
    pwm_duty=error*kp+pwm_duty;                         //Set the corrective PWM to output
    if(pwm_duty<0)  pwm_duty=0;
    if(pwm_duty>MAX_DUTY) pwm_duty=MAX_DUTY;
    Timer1.pwm(PWM_OUT, (int)pwm_duty);
    Serial.print("Voltage:");
    Serial.print(voltage);
    Serial.print("V Error:");
    Serial.print(error);
    Serial.print("V Duty:");
    Serial.println(pwm_duty);
}

